# Copyright (C) 2003 Paramjit Oberoi <param@cs.wisc.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk.glade
import os, os.path
import ConfigParser

def unparse_prefixes(prefixes):
    result = ','.join(prefixes)
    if not result: return ""
    # check for spaces at begin/end and insert extra commas if needed
    if result[0] == " ": result = ","+result
    if result[-1] == " ": result = result + ","
    return result

def parse_prefixes(s):
    prefixes = []
    cur_prefix = ""
    saw_comma = False
    for c in s:
        if c == ",":
            if saw_comma:
                saw_comma = False
                cur_prefix += c
            else:
                saw_comma = True
        else:
            if saw_comma:
                if cur_prefix: prefixes.append(cur_prefix)
                saw_comma = False
                cur_prefix = c
            else:
                cur_prefix += c

    if cur_prefix: prefixes.append(cur_prefix)
    return prefixes

class MoosicAppletConfig:
    def __init__(self, moo, config_file=None):
        self.moo = moo
        self.filename = config_file
        if not self.filename:
            self.filename = os.path.expanduser("~/.moosicappletrc")
            if self.filename[0] == "~": self.filename = ".moosicappletrc"

        self.defaults()
        self.load_file()

    def defaults(self):
        self.display_qlength = True
        self.display_loop = False
        self.width = 150
        self.strip_dir = True
        self.strip_ext = True
        self.replace_underscores = True
        self.strip_prefix = False
        self.prefixes = ["--"," - "]

    def load_file(self):
        c = ConfigParser.ConfigParser()
        c.read(self.filename)
        try: self.display_qlength = c.getboolean("ui","display_qlength")
        except: pass
        try: self.display_loop = c.getboolean("ui","display_loop")
        except: pass
        try: self.width = c.getint("ui","width")
        except: pass
        try: self.strip_dir = c.getboolean("songname","strip_dir")
        except: pass
        try: self.strip_ext = c.getboolean("songname","strip_ext")
        except: pass
        try: self.replace_underscores = c.getboolean("songname","replace_underscores")
        except: pass
        try: self.strip_prefix = c.getboolean("songname","strip_prefix")
        except: pass
        try: self.prefixes = parse_prefixes(c.get("songname","prefixes"))
        except: pass

    def save_file(self):
        c = ConfigParser.ConfigParser()
        c.add_section("ui")
        c.add_section("songname")
        c.set("ui","display_qlength",self.display_qlength)
        c.set("ui","display_loop",self.display_loop)
        c.set("ui","width",self.width)
        c.set("songname","strip_dir",self.strip_dir)
        c.set("songname","strip_ext",self.strip_ext)
        c.set("songname","replace_underscores",self.replace_underscores)
        c.set("songname","strip_prefix",self.strip_prefix)
        c.set("songname","prefixes",unparse_prefixes(self.prefixes))
        c.write(file(self.filename,"w+"))

    def prefs_ui(self):
        # def tmp(*args):
        #     print "tmp called with",len(args),"arguments"
        #     for i in args: print type(i), i

        def on_close_button_clicked(button):
            self.save_file()
            xml.get_widget('prefs_window').destroy()

        def on_display_qlength_toggled(togglebutton):
            self.display_qlength = togglebutton.get_active()
            self.moo.queue_length = None  # to ensure display refresh
            if self.display_qlength:
                self.moo.ui.create_qlength_display()
            else:
                self.moo.ui.destroy_qlength_display()
            self.moo.ui.hbox.show_all()

        def on_display_loop_toggled(togglebutton):
            self.display_loop = togglebutton.get_active()
            self.moo.looping = None  # to ensure display refresh
            if self.display_loop:
                self.moo.ui.create_loop_display()
            else:
                self.moo.ui.destroy_loop_display()
            self.moo.ui.hbox.show_all()

        def on_song_width_value_changed(spinbutton):
            self.width = spinbutton.get_value_as_int()
            self.moo.ui.label.set_size_request(self.width,-1);

        def on_strip_dir_toggled(togglebutton):
            self.strip_dir = togglebutton.get_active()
            self.moo.current_song = None

        def on_strip_ext_toggled(togglebutton):
            self.strip_ext = togglebutton.get_active()
            self.moo.current_song = None

        def on_replace_underscores_toggled(togglebutton):
            self.replace_underscores = togglebutton.get_active()
            self.moo.current_song = None

        def on_strip_prefix_toggled(togglebutton):
            self.strip_prefix = togglebutton.get_active()
            self.moo.current_song = None

        def on_prefixes_changed(gtkentry):
            self.prefixes = parse_prefixes(gtkentry.get_text())
            self.moo.current_song = None

        def on_defaults_button_clicked(button):
            self.defaults()
            load_values_into_ui()
            self.moo.current_song = None

        def load_values_into_ui():
            xml.get_widget('display_qlength').set_active(self.display_qlength)
            xml.get_widget('display_loop').set_active(self.display_loop)
            xml.get_widget('song_width').set_value(self.width)
            xml.get_widget('strip_dir').set_active(self.strip_dir)
            xml.get_widget('strip_ext').set_active(self.strip_ext)
            xml.get_widget('replace_underscores').set_active(self.replace_underscores)
            xml.get_widget('strip_prefix').set_active(self.strip_prefix)
            xml.get_widget('prefixes').set_text(unparse_prefixes(self.prefixes))

        #xml = gtk.glade.XML("<path>/prefs-window.glade")

        # The following magic depends on Fredrik Lundh's "squeezeTool" -- all files
        # required for the applet are packaged up into just one file at build time.
        # __main__.open() returns a stringIO file object

        import __main__
        s = __main__.open("prefs-window.glade").read()
        xml = gtk.glade.xml_new_from_buffer(s,len(s))

        xml.signal_autoconnect(locals())
        load_values_into_ui()

        xml.get_widget('close_button').grab_focus()
        xml.get_widget('close_button').grab_default()




# applet.py - A GNOME applet that shows what Moosic is currently playing.
#
# Copyright (C) 2003 Daniel Pearson <daniel.pearson@umbc.edu>
# Copyright (C) 2003 Paramjit Oberoi <param@cs.wisc.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import pygtk
pygtk.require('2.0')
import gtk, gnome.applet

import socket, errno, os, os.path, sys
import xmlrpclib, moosic_factory

from applet_ui import *
from applet_config import *

# Class representing the applet
# An instance of this class is instantiated for each applet

class MoosicApplet:
    def refresh(self):
        call_array = [{'methodName':'length','params':[]},
                      {'methodName':'current','params':[]},
                      {'methodName':'is_paused','params':[]},
                      {'methodName':'is_queue_running','params':[]},
                      {'methodName':'list','params':[[0,1]]},
                      {'methodName':'is_looping','params':[]},
                     ]
        try: result_array = self.moosic.system.multicall(call_array)
        except socket.error, e:
            if e[0] in (errno.ECONNREFUSED, errno.ENOENT):
                self.ui.update_tooltip('[moosicd not running]')
            else:
                self.ui.update_tooltip('[unexpected socket error]')
            self.update_song_name("[not connected]", False)
            self.connected = False
            return gtk.TRUE

        # # Make sure that our connection to the server is working properly.
        # try: self.moosic.no_op()
        # except socket.error, e:
        #     if e[0] in (errno.ECONNREFUSED, errno.ENOENT):
        #         self.ui.update_tooltip('[moosicd not running]')
        #     else:
        #         self.ui.update_tooltip('[unexpected socket error]')
        #     self.update_song_name("[not connected]", False)
        #     self.connected = False
        #     return gtk.TRUE

        update_display = False
        if not self.connected:
            self.connected = True
            update_display = True

        # current_song = self.moosic.current().data.decode('latin-1').encode('utf-8')
        # paused = self.moosic.is_paused()
        # stopped = not current_song
        # queue_length = self.moosic.length()
        # queue_running = self.moosic.is_queue_running()

        current_song = result_array[1][0].data.decode('latin-1').encode('utf-8')
        paused = result_array[2][0]
        looping = result_array[5][0]
        stopped = not current_song
        queue_length = result_array[0][0]
        queue_running = result_array[3][0]
        if queue_length:
            first_in_queue = result_array[4][0][0].data.decode('latin-1').encode('utf-8')
        else:
            first_in_queue = None

        if self.paused != paused:
            self.paused = paused
            if paused: self.ui.paused()
            else: self.ui.unpaused()
            update_display = True

        if self.stopped != stopped:
            self.stopped = stopped;
            update_display = True

        if self.queue_running != queue_running:
            self.queue_running = queue_running
            if queue_running: self.ui.queue_running()
            else: self.ui.queue_not_running()

        if self.looping != looping:
            self.looping = looping
            if looping: self.ui.loop_mode_enabled()
            else: self.ui.loop_mode_disabled()

        if self.queue_length != queue_length:
            self.queue_length = queue_length
            if self.config.display_qlength:
                #self.ui.qlen_label.set_text("%d" % queue_length)
                self.ui.qlen_label.set_markup("<b>%d</b>" % queue_length)
            update_display = True

        if self.current_song != current_song:
            self.current_song = current_song
            update_display = True

        if self.first_in_queue != first_in_queue:
            self.first_in_queue = first_in_queue
            update_display = True

        if update_display:
            # update stop/play/add button
            if stopped:
                if self.queue_length: self.ui.stopped()
                else: self.ui.empty()
            else: self.ui.playing()

            # update song name and tooltip
            if stopped:
                if queue_length:
                    self.ui.update_tooltip("[next item] " + first_in_queue)
                else:
                    self.ui.update_tooltip("[queue empty]")
                if self.config.display_qlength or queue_length:
                    self.update_song_name("[nothing playing]", False)
                else:
                    self.update_song_name("[queue empty]", False)
            elif paused:
                self.ui.update_tooltip("[paused] " + current_song)
                self.update_song_name(current_song, True)
            else:
                self.ui.update_tooltip(current_song)
                self.update_song_name(current_song, True)

        #if self.moosic.is_looping(): is_looping="1"
        #else: is_looping = "0"
        #self.menu.set_property("/commands/Loop", {"state":is_looping})
        #print self.menu.get_prop("/commands/Loop")

        #pdb.set_trace()

        return gtk.TRUE

    def update_song_name(self, s, munge):
        if munge:
            if (self.config.strip_dir): s = os.path.basename(s)
            if (self.config.strip_ext): s = os.path.splitext(s)[0]
            if (self.config.strip_prefix):
                dirname, filename = os.path.split(s)
                for pfx in self.config.prefixes:
                    n = filename.rfind(pfx)
                    if n != -1: filename = filename[n+len(pfx):]
                s = os.path.join(dirname,filename)
            if (self.config.replace_underscores): s = s.replace("_"," ")
            s = s.strip()

        self.ui.label.set_text(s)

    # ---- Utility functions for controlling moosicd

    def construct_file_list(self, lst, result=None):
        if result is None: item_list = []
        else: item_list = result
        for item in lst:
            if not os.path.isdir(item):
                item_list.append(xmlrpclib.Binary(item))
            else:
                dirlist = os.listdir(item)
                # ignore dotfiles and dotdirs?
                dirlist = [os.path.join(item,x) for x in dirlist]
                self.construct_file_list(dirlist, item_list)
        return item_list

    def add(self, run_queue=False):
        def callback(dlg,response):
            if response==gtk.RESPONSE_OK:
                selections = dlg.get_selections()
                self.moosic.append(self.construct_file_list(selections))
                if run_queue: self.moosic.run_queue()
                last_sel = selections[-1]
                if not os.path.isdir(last_sel):
                    last_sel = os.path.dirname(last_sel)
                os.chdir(last_sel)
            dlg.destroy()

        f = gtk.FileSelection("Add Entries to Moosic Queue")
        f.set_select_multiple(True)
        f.hide_fileop_buttons()
        f.connect("response",callback)
        f.show_all()


    def play_stop_add(self):
        if self.stopped:
            if not self.queue_length: self.add(True)
            else: self.moosic.run_queue()
        else:
            self.moosic.stop()

    def pause_toggle(self):
        self.moosic.toggle_pause()

    def prev(self):
        self.moosic.previous()

    def next(self):
        self.moosic.next()

    def sort(self):
        self.moosic.sort()

    def shuffle(self):
        self.moosic.shuffle()

    def qstop(self):
        self.moosic.halt_queue()

    def clear(self):
        self.moosic.clear()

    def wipe(self):
        self.moosic.halt_queue()
        self.moosic.next()
        self.moosic.clear()

    def toggle_queue_running(self):
        if self.queue_running:
            self.moosic.halt_queue()
        else:
            self.moosic.run_queue()

    def toggle_loop_mode(self):
        self.moosic.toggle_loop_mode()

    def __init__(self, applet, iid):
        self.stopped = True
        self.paused = False
        self.looping = None
        self.queue_running = False
        self.queue_length = None
        self.current_song = None
        self.first_in_queue = None
        self.connected = False

        self.moosic = moosic_factory.LocalMoosicProxy()
        self.config = MoosicAppletConfig(self)
        self.ui = MoosicAppletUI(applet, self)


def moosic_applet_factory(applet, iid):
    MoosicApplet(applet,iid)
    return gtk.TRUE

# Run as application?
if len(sys.argv) == 2 and sys.argv[1] == "run-moosicapplet-as-application":
    main_window = gtk.Window(gtk.WINDOW_TOPLEVEL)
    main_window.set_title("Moosic Applet")
    main_window.connect("destroy", gtk.mainquit) # quit the event loop on destruction
    app = gnome.applet.Applet()
    moosic_applet_factory(app, None)
    app.reparent(main_window)
    main_window.show_all()
    gtk.main()
    sys.exit()

gnome.applet.bonobo_factory("OAFIID:GNOME_MoosicApplet_Factory", gnome.applet.Applet.__gtype__, "hello", "0", moosic_applet_factory)



# Copyright (C) 2003 Daniel Pearson <daniel.pearson@umbc.edu>
# Copyright (C) 2003 Paramjit Oberoi <param@cs.wisc.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk
import gnome.applet

import socket, errno, os.path

# Register Icons

import icons

# The icons.py module contains string representations of
# icons, created using the "gdk-pixbuf-csource" program.
# The namespace of the module is scanned, and all the
# available icons are registered as stock icons so that
# they can be referenced simply using the icon name.
#
# The gnome convention for icon names is to use dashes
# as in "gtk-clear" or "media-pause".  Python variables
# can't contain dashes, so icons.py contains underscores
# instead, and these are replaced with dashes when the
# icons are registered.

factory = gtk.IconFactory()
factory.add_default()
for k in icons.__dict__.keys():
    if k.startswith("__"): continue
    img = icons.__dict__[k]
    name = "moo-"+k.replace("_","-")
    pixbuf = gtk.gdk.pixbuf_new_from_inline(len(img), img, 0)
    iconset = gtk.IconSet(pixbuf)
    #print "registering icon", name
    factory.add(name, iconset)


class MoosicAppletUI:

    def paused(self):
        self.pause_toggle_icon.set_from_stock('moo-media-pause-red',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.pause_toggle_button, 'Unpause')

    def unpaused(self):
        self.pause_toggle_icon.set_from_stock('moo-media-pause',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.pause_toggle_button, 'Pause')

    def stopped(self):
        self.play_stop_add_icon.set_from_stock('moo-media-play',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.play_stop_add_button, 'Play')

    def playing(self):
        self.play_stop_add_icon.set_from_stock('moo-media-stop',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.play_stop_add_button, 'Stop')

    def empty(self):
        self.play_stop_add_icon.set_from_stock('moo-add',gtk.ICON_SIZE_BUTTON)
        self.tooltips.set_tip(self.play_stop_add_button, 'Add')

    def update_tooltip(self, s):
        #self.tooltips.set_tip(self.app_parent, s)
        #self.tooltips.set_tip(self.applet, s)
        self.tooltips.set_tip(self.ebox, s)

    def queue_running(self):
        self.qstatus_icon.set_from_stock('gtk-yes',gtk.ICON_SIZE_BUTTON)

    def queue_not_running(self):
        self.qstatus_icon.set_from_stock('gtk-no',gtk.ICON_SIZE_BUTTON)

    def loop_mode_enabled(self):
        if self.loop_ebox:
            self.loop_icon.set_from_stock('moo-loop-red',gtk.ICON_SIZE_BUTTON)

    def loop_mode_disabled(self):
        if self.loop_ebox:
            self.loop_icon.set_from_stock('moo-loop-gray',gtk.ICON_SIZE_BUTTON)

    # ---- Buttons

    def create_buttons(self, hbox):
        Buttons = []
        Buttons += [(self.moo.play_stop_add,  'moo-media-play',   'Play')]
        Buttons += [(self.moo.pause_toggle, 'moo-media-pause',  'Pause')]
        Buttons += [(self.moo.prev, 'moo-media-rewind', 'Previous')]
        Buttons += [(self.moo.next, 'moo-media-ffwd',   'Next')]

        for (func,icon, tip) in Buttons:
            button = gtk.Button()
            buttonicon = gtk.image_new_from_stock(icon, gtk.ICON_SIZE_BUTTON)
            button.add(buttonicon)
            # size_request(0) makes the button as small as possible
            # otherwise the button is too large with gtk-2.4
            # (but, this also prevents buttons from expanding
            # automatically if the panel is larger)
            button.set_size_request(-1,0)
            hbox.add(button)
            button.connect("clicked", lambda btn,f=func: f())
            self.__dict__[func.__name__ + "_button"] = button
            self.__dict__[func.__name__ + "_icon"] = buttonicon
            self.tooltips.set_tip(button, tip)

    # ---- Menu entries

    def create_menu(self):
        Menuitems = []
        Menuitems += [(self.moo.add, '_Add Items', 'moo-add')]
        Menuitems += [(self.moo.sort, '_Sort Items', 'gtk-sort-ascending')]
        Menuitems += [(self.moo.shuffle, 'Shu_ffle Items', 'gtk-refresh')]
        Menuitems += [(self.moo.clear, '_Clear Queue', 'gtk-clear')]
        Menuitems += [(self.moo.wipe, 'Clear _Queue and Stop', 'gtk-stop')]
        Menuitems += [(None, None, None)]  # seperator

        # def toggle_loop(self, *args):
        #     print "zz"
        #     self.menu.get_property("Loop")
        # Menuitems += [(toggle_loop, 'toggle_loop', 'gtk-help')]

        Menuitems += [(self.moo.config.prefs_ui, '_Preferences', 'gtk-preferences')]

        # Add entries to the applet's popup menu.
        verbs = []
        xml_template = """<menuitem name="%(name)s" verb="%(name)s" _label="%(label)s"
                                    pixtype="stock" pixname="%(icon-name)s" />"""
        menu_xml = '<popup name="button3">'
        for (func,label,icon) in Menuitems:
            if not func:
                menu_xml += "<separator/>"
            else:
                menu_xml += xml_template % {
                                    "name" : func.__name__,
                                    "label": label,
                                    "icon-name": icon }
                verbs += [(func.__name__, lambda gob,mname,f=func: f())]

        # def loop_verb(gob, name):
        #     print "loop_verb called with",len(args),"arguments"
        #     for i in args: print type(i), i
        #     #import pdb
        #     #pdb.set_trace()
        #
        # self.loop_verb = 0
        # menu_xml += '<menuitem name="Loop" verb="Loop" _label="_Loop" type="toggle" />'
        # verbs += [("Loop", loop_verb)]

        menu_xml += '</popup>'
        self.applet.setup_menu(menu_xml, verbs, None)

    def create_qlength_display(self):
        if not self.qlen_ebox:
            self.qlen_label = gtk.Label()
            self.qlen_label.set_padding(2,0)
            self.qlen_ebox = gtk.EventBox()
            self.qlen_ebox.add(self.qlen_label)
            self.tooltips.set_tip(self.qlen_ebox, "Queue Length")
            self.hbox.add(self.qlen_ebox)
            if self.loop_ebox: self.hbox.reorder_child(self.qlen_ebox,2)
            else: self.hbox.reorder_child(self.qlen_ebox,1)

    def destroy_qlength_display(self):
        if (self.qlen_ebox):
            self.qlen_ebox.destroy()
            self.qlen_ebox = self.qlen_label = None

    def qstatus_icon_clicked(self, ebox, event):
        if event.button == 1:
            self.moo.toggle_queue_running()

    def create_loop_display(self):
        if not self.loop_ebox:
            self.loop_icon = gtk.image_new_from_stock('moo-loop-gray',gtk.ICON_SIZE_BUTTON)
            self.loop_ebox = gtk.EventBox()
            self.loop_ebox.add(self.loop_icon)
            self.loop_ebox.connect("button-press-event",self.loop_icon_clicked)
            self.tooltips.set_tip(self.loop_ebox, "Enable/Disable Loop Mode")
            self.hbox.add(self.loop_ebox)
            self.hbox.reorder_child(self.loop_ebox,1)

    def destroy_loop_display(self):
        if (self.loop_ebox):
            self.loop_ebox.destroy()
            self.loop_ebox = self.loop_label = None

    def loop_icon_clicked(self, ebox, event):
        if event.button == 1:
            self.moo.toggle_loop_mode()

    # ---- Initialize the applet.

    # Widget tree is as follows:
    #
    #     self.applet
    #     |
    #     +--self.frame (for border)
    #        |
    #        +-- self.hbox
    #            |
    #            +--self.qstatus_ebox (to catch clicks on qstatus_icon, and tooltips)
    #            |  |
    #            |  +--self.qstatus_icon
    #            |
    #            +--self.qlen_ebox (queue length tooltip)
    #            |  |
    #            |  +--self.qlen_label
    #            |
    #            +--self.ebox (EventBox for a song-name tooltip)
    #            |  |
    #            |  +--hbox2
    #            |     |
    #            |     +--label
    #            |     |
    #            |     +--bogus-label-for-padding
    #            |
    #            +--Button1
    #            |
    #            +--Button2
    #            |
    #            +--<more buttons...>

    # Each button has a tooltip; self.ebox has a tooltip as well.
    # self.ebox exists only for the tooltip; assigning that tooltip
    # to self.applet leads to the tooltip not being displayed when
    # the mouse is moved off one of the buttons onto other areas.

    def __init__(self, applet, moo):
        self.applet = applet
        self.moo = moo
        self.frame = gtk.Frame()
        # Choose from ETCHED_IN / ETCHED_OUT / IN / OUT / NONE
        self.frame.set_shadow_type(gtk.SHADOW_ETCHED_IN)
        applet.add(self.frame)

        self.tooltips = gtk.Tooltips()

        self.hbox = gtk.HBox()
        self.frame.add(self.hbox)

        # Queue status
        self.qstatus_ebox = gtk.EventBox()
        self.qstatus_icon = gtk.image_new_from_stock('gtk-no',gtk.ICON_SIZE_BUTTON)
        self.qstatus_ebox.connect("button-press-event",self.qstatus_icon_clicked)
        self.tooltips.set_tip(self.qstatus_ebox, "Start/Stop Queue")
        self.qstatus_ebox.add(self.qstatus_icon)
        self.hbox.add(self.qstatus_ebox)

        # Looping
        self.loop_ebox = self.loop_label = None
        if self.moo.config.display_loop:
            self.create_loop_display()

        # Queue length
        self.qlen_ebox = self.qlen_label = None
        if self.moo.config.display_qlength:
            self.create_qlength_display()

        #self.hbox.add(gtk.image_new_from_stock('moo-loop-red',gtk.ICON_SIZE_BUTTON))
        #self.hbox.add(gtk.image_new_from_stock('moo-loop-gray',gtk.ICON_SIZE_BUTTON))

        self.ebox = gtk.EventBox()
        self.hbox.add(self.ebox)
        hbox2 = gtk.HBox()
        self.ebox.add(hbox2)

        # Current Song
        self.label = gtk.Label("")
        self.label.set_padding(5,0)
        self.label.set_size_request(self.moo.config.width,-1)
        hbox2.add(self.label)

        # dummy label for padding
        # (there must be a better way of doing this)
        # (the alignment widget?)
        l = gtk.Label("")
        l.set_padding(3,0)
        hbox2.add(l)

        #self.app_parent = applet.get_parent()

        self.create_buttons(self.hbox)

        self.create_menu()

        # self.menu = applet.get_popup_component()
        # print self.menu
        # print type(self.menu)
        # print dir(self.menu)

        gtk.timeout_add(500, moo.refresh)

        def bye(*args):
            # this is needed to avoid warnings about leaked objects
            del self.applet
        applet.connect("destroy",bye)

        #pdb.set_trace()

        # How to add a handle to the applet?
        # 4 is supposed to denote the flag PANEL_APPLET_HAS_HANDLE, but it doesn't work
        # (there is also a 'handlebutton' I believe...)
        # self.applet.set_flags(4)

        applet.show_all()




# distutils based setup script
# for making tarballs to ease distribution

from distutils.core import setup

setup(name="moosic-applet",
      version="0.0.4",
      description="Gnome applet for controlling Moosic",
      author="Paramjit Oberoi and Daniel Pearson",
      author_email="param@cs.wisc.edu",
      url="http://www.nanoo.org/~daniel/moosic/",
      license="GPL",
     )



# moosic_factory.py - Factory functions for creating Moosic server proxies.
#
# Copyright (C) 2001-2003 Daniel Pearson <daniel.pearson@umbc.edu>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""Factory functions for creating Moosic server proxies.

This module contains functions that make it very quick and easy to create a
proxy object that exposes all the methods of a Moosic server.  These proxy
objects are instances of xmlrpclib.ServerProxy, so you should refer to the
documentation for the xmlrpclib module for more detailed information about
ServerProxy objects.  These factory functions are not necessary for creating
proper Moosic server proxies, but they are a very convenient way of doing so.

This module also contains a subclass of xmlrpclib.Transport that adapts the
XML-RPC client implementation for use with Unix sockets.

It is safe to "import *" from this module.
"""

import xmlrpclib, urllib, httplib, socket, os, os.path, popen2, sys

# Define the True and False constants if they don't already exist.
try: True
except NameError: True = 1
try: False
except NameError: False = 0


__all__ = ('UnixStreamTransport', 'LocalMoosicProxy',
           'UnixMoosicProxy', 'InetMoosicProxy', 'startServer')


class UnixStreamTransport(xmlrpclib.Transport):
    '''An adapter for speaking HTTP over a Unix socket instead of a TCP/IP socket.
    '''
    def make_connection(self, host):
        class HTTPConnection(httplib.HTTPConnection):
            def connect(self):
                host = urllib.unquote(self.host)
                try:
                    self.sock = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
                    if self.debuglevel > 0:
                        print "connect: (%s)" % host
                    self.sock.connect(host)
                except socket.error, msg:
                    if self.debuglevel > 0:
                        print 'connect fail:', host
                    if self.sock:
                        self.sock.close()
                    self.sock = None
                    raise
        class HTTP(httplib.HTTP):
            _connection_class = HTTPConnection
        return HTTP(host)


def LocalMoosicProxy(filename=None):
    '''Creates a proxy to a Moosic server that is listening to a local (Unix
    address family) socket.
    
    The optional "filename" argument is the location of the socket file that the
    Moosic server is using as its address.
    '''

    if not filename:
        filename = os.path.join(os.getenv('HOME', '/tmp'), '.moosic',
                                'socket-' + socket.gethostname())
    server_address = 'http://%s/' % urllib.quote(filename, safe='')
    return xmlrpclib.ServerProxy(uri=server_address,
                                 transport=UnixStreamTransport(),
                                 verbose=False)


def UnixMoosicProxy(filename=None):
    '''An alias for the LocalMoosicProxy function.
    '''
    return LocalMoosicProxy(filename)


def InetMoosicProxy(host, port):
    '''Creates a proxy to a Moosic server that is listening to a TCP/IP socket.
    
    The first argument, "host", is the hostname or IP address where the server
    is located. The second argument, "port", is the TCP port that the server is
    listening to.
    '''
    return xmlrpclib.ServerProxy(uri='http://%s:%d/' % (host, port),
                                 verbose=False)


def startServer(config_dir):
    '''Attempts to run the "moosicd" program.
    
    The location of the program is first sought in the MOOSICD environment
    variable (which should contain the full pathname of the program), and then
    sought in the directories listed in the PATH environment variable.
    
    The "config_dir" parameter is the path name of the directory where moosicd
    should locate the files it uses at runtime. This is usually set to the
    ".moosic" directory in the user's home directory.
    
    If moosicd can't be started, a string describing why is returned.
    Otherwise, None is returned.
    '''
    # Use the MOOSICD environment variable if it is set.
    moosicd = os.getenv('MOOSICD', '')
    if moosicd and not (os.access(moosicd, os.R_OK | os.X_OK) and os.path.isfile(moosicd)):
        print >>sys.stderr, 'Warning:', moosicd, 'is not an executable file.'
        print >>sys.stderr, 'Ignoring the value of $MOOSICD and searching $PATH instead.'
        moosicd = None
    # Check whether moosicd is in the execution path.
    if not moosicd:
        for dir in os.getenv('PATH', ':/bin:/usr/bin').split(':'):
            f = os.path.join(dir, 'moosicd')
            if os.access(f, os.R_OK | os.X_OK) and os.path.isfile(f):
                moosicd = f
                break
    # Search for a program named "MoosicDaemon" if a program named "moosicd"
    # wasn't found.
    if not moosicd:
        for dir in os.getenv('PATH', ':/bin:/usr/bin').split(':'):
            f = os.path.join(dir, 'MoosicDaemon')
            if os.access(f, os.R_OK | os.X_OK) and os.path.isfile(f):
                moosicd = f
                break
    if not moosicd:
        return 'The "moosicd" program could not be found.'
    # Execute moosicd.
    import commands
    status, output = commands.getstatusoutput("%s -c %s" % (moosicd, config_dir))
    if os.WIFEXITED(status):
        status = os.WEXITSTATUS(status)
    else:
        return "moosicd did something rather abnormal."
    if status == 0:
        return
    else:
        return output + "\nExit status: %d" % status

